--[[
This class represents a specific set of units of the same type (player, party,
pet, partypet, raid, target, etc.).

A UnitGroup has a single set of Action instances.
A UnitGroup has one or more UnitGroupHeader instances.  How many it has depends
on WoW, unit frame add-ons, and group configuration.  WoW 2.01 has a bug with
frame-unit associations that force a UnitGroupHeader instance for each unit
frame (WoW 2.02 should fix this)..
--]]
uab.classes.HostileActions = uab.mixins.aceOO.Class()

function uab.classes.HostileActions.prototype:init(activator, defn)
	uab.classes.HostileActions.super.prototype.init(self)

	self.name = "hostile"
	self.activator = activator
	self.qualifiedname = self.activator.name .. self.name
	self.actions = {}
	self.defn = defn

    if self.defn.actions then
    	for idx = 1, 24, 1 do
            local action = self.defn.actions[idx]
    		if action then
                if action.classname then
    		        self.actions[idx] = uabAction:CreateAction(idx, action, true)
                else
                    self.defn.actions[idx] = nil
                end
    		end
    	end
    else
        self.defn.actions = {}
    end
end

function uab.classes.HostileActions.prototype:HasAnyActions()
    for idx = 1, 24, 1 do
        if self.defn.actions[idx] then
            return true
        end
    end
end

function uab.classes.HostileActions.prototype:CopyTo(target)
    --[[
    Copy all the actions to the target group.  This means duplicating the action, then
    assigning the action to the same spot in the target group.  It also means that if
    we do not have an action for a specific position, we must clear the action for that
    position in the target group as well.
    --]]

    for idx = 1, 24, 1 do
        local data = nil
        
        if self.defn.actions[idx] then
            data = uab.CloneTable(self.defn.actions[idx])
        end

        target:SetAction(idx, data)
    end
end

function uab.classes.HostileActions.prototype:GetAction(number)
    return self.defn.actions[number] and self.actions[number]
end

function uab.classes.HostileActions.prototype:SetAction(number, data)
    if data and data.classname then
    	self.defn.actions[number] = data
    	
    	if self.actions[number] then
    		local actiondata = uab.classes[data.classname]:new(data.data)
    	    
            self.actions[number]:SetAll(actiondata, data.effect)
    	else
    	    self.actions[number] = uabAction:CreateAction(number, data, true)
    	end
        self.activator:HostileActionsChanged()
    else
        self:ClearAction(number)
    end
end

function uab.classes.HostileActions.prototype:ClearAction(number)
    self.defn.actions[number] = nil
    if self.actions[number] then
        self.actions[number]:Clear()
    end
    self.activator:HostileActionsChanged()
end

function uab.classes.HostileActions.prototype:FindOnlyAction()
    local onlyaction

    for idx = 1, 24, 1 do
        if self.defn.actions[idx] then
            if onlyaction then
                return nil
            end

            onlyaction = self.actions[idx]
        end
    end
    
    return onlyaction
end
