uab.classes.ItemAction = uab.mixins.aceOO.Class(uab.classes.BaseAction)

function uab.classes.ItemAction.prototype:init(data)
	uab.classes.ItemAction.super.prototype.init(self)
	
	self.data = data
    self.text = self.data.name
    self.attribs = {}

    self.attribs["type"] = "item"
    self.attribs["item"] = self.text
    
	self:SetIcon()
end

function uab.classes.ItemAction.prototype:GetAttributes()
    return self.attribs
end

function uab.classes.ItemAction.prototype:GetClassName()
	return "ItemAction"
end

function uab.classes.ItemAction.prototype:IsUsable()
    return IsUsableItem(self:GetText())
end

function uab.classes.ItemAction.prototype:GetCooldown()
    return GetItemCooldown(self.text)
end

function uab.classes.ItemAction.prototype:GetCount()
    local bag, slot = self:FindItem()

    if bag and slot then
        local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bag, slot)
        
        if itemCount then
            return itemCount
        else
            local _, _, _, _, _, _, _, maxstack, _, _ = GetItemInfo(self.text)
  
            if maxstack > 1 then
                return "0"
            end
        end
    elseif not slot then
        --[[
        Need to handle items that are no longer in inventory or bags.  In this case
        need to either return a nil value or a count of 0 based on whether the item
        is equipable or not, or a non-stackable item.
        --]]

        if self.data.itemlink then
            local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemCount, itemEquipLoc, itemTexture = GetItemInfo(self.data.itemlink)

            if itemType == "Armor" or itemType == "Container" or itemType == "Recipe" or itemType == "Quest" or itemType == "Quiver" or itemType == "Weapon" then
                return
            end
        
            if itemType == "Consumable" or itemType == "Key" or itemType == "Miscellaneous" or itemType == "Reagent" or itemType == "Projectile" or itemType == "Trade Goods" then
                return "0"
            end
        end
    end
end

function uab.classes.ItemAction.prototype:ShowTooltip(frame)
    --[[
    Need to find the referenced item.  Look in the player's inventory first.
    If not there, look in the player's bags.
    --]]

    GameTooltip:SetOwner(frame, "ANCHOR_RIGHT")
    GameTooltip:SetBackdropColor(0.1, 0.1, 0.1, 0.8)
    
    local bag, slot = self:FindItem()

    if bag and slot then
        GameTooltip:SetBagItem(bag,slot)
    elseif slot then
        GameTooltip:SetInventoryItem("player", slot)
    elseif self.data.itemlink then
        GameTooltip:SetHyperlink(self.data.itemlink)
    end
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.ItemAction.prototype:SetIcon()
    self.icon = self.data.icon

    if not self.icon then
    	--[[
    	Need to find the referenced item.  Look in the player's inventory first.
        If not there, look in the player's bags.
    	--]]
    
        local bag, slot = self:FindItem()
    
        if bag and slot then
            self.icon = GetContainerItemInfo(bag,slot)
            self.data.itemlink = GetContainerItemLink(bag,slot)
        elseif slot then
            self.icon = GetInventoryItemTexture("player",slot)
            self.data.itemlink = GetInventoryItemLink("player",slot)
        else
            local _, link, _, _, _, _, _, _, _, texture = GetItemInfo(self.text)
    
            self.icon = texture
            self.data.itemlink = link
        end
    end        
    
    self.data.icon = self.icon
end

function uab.classes.ItemAction.prototype:FindItem()
    for bag=0,4 do
        for slot=1,GetContainerNumSlots(bag) do
            if GetContainerItemLink(bag,slot) then
                if string.find(GetContainerItemLink(bag,slot), self.text) then
                    return bag, slot
                end
            end
        end
    end
    
    for slot = 0, 19, 1 do
        if GetInventoryItemLink("player", slot) then
            if string.find(GetInventoryItemLink("player", slot), self.text) then
                return nil, slot
            end
        end
    end
end

