local uabBlizzardFrameGroup = uabFrameGroup:NewModule("Blizzard")

uabBlizzardFrameGroup.controls = {}
uabBlizzardFrameGroup.defaultDB = {
    framegroups = {
        ["Player"] = {
            expression = "^PlayerFrame$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "player",
        },
        ["Player Pet"] = {
            expression = "^PetFrame$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "pet",
        },
        ["Party"] = {
            expression = "^PartyMemberFrame.+",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "partyraid",
        },
        ["Target"] = {
            expression = "^TargetFrame$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "target",
        },
        ["Target of Target"] = {
            expression = "^TargetofTargetFrame$",
            usedefaultscale = true,
            usedefaultanchor = true,
            unitgroup = "targettarget",
        },
    },
}

function uabBlizzardFrameGroup:OnInitialize()
    self.super.OnInitialize(self)
end

function uabBlizzardFrameGroup:OnEnable()
end

function uabBlizzardFrameGroup:OnDisable()
end

function uabBlizzardFrameGroup:NewActivator(activator)
	self.controls[activator] = {}
	
    self:AllocateControlsForUnitGroup(activator, "player")
    self:AllocateControlsForUnitGroup(activator, "pet")
    self:AllocateControlsForUnitGroup(activator, "partyraid")
    self:AllocateControlsForUnitGroup(activator, "target")
    self:AllocateControlsForUnitGroup(activator, "targettarget")
end

function uabBlizzardFrameGroup:AllocateControlsForUnitGroup(activator, groupname)
    self.controls[activator][groupname] = {}
	for idx = 1, uab.constants.groupInfo[groupname].unitcount do
        table.insert(self.controls[activator][groupname], activator:NewUnitGroupControl(groupname))
    end
end

function uabBlizzardFrameGroup:HaveFrameControl(frame, activator)
	if self.db.profile.disabled then return end

	local _, options = self:OptionsForFrame(frame)
	
	if not options or options.disabled then return end

	if self.controls[activator] and self.controls[activator][options.unitgroup] then
		return # self.controls[activator][options.unitgroup] > 0
	end
end

function uabBlizzardFrameGroup:NextFrameControl(frame, activator)
	if self.db.profile.disabled then return end

	local _, options = self:OptionsForFrame(frame)

	if not options or options.disabled then return end
	
    if self.controls[activator] and self.controls[activator][options.unitgroup]
        and # self.controls[activator][options.unitgroup] > 0 then
		control = table.remove(self.controls[activator][options.unitgroup])
	end

	return control
end

function uabBlizzardFrameGroup:OptionsForFrame(frame)
    if self.db.profile.disabled then return end
    
    for name, defn in pairs(self.db.profile.framegroups) do
        if self:MatchFrame(frame, defn) then
            return name, defn
        end
    end
end

function uabBlizzardFrameGroup:CreateFrameGroupOptions()
    local args = {}

    args = {
        type = 'group',
        name = "Blizzard",
        desc = "Use, anchor, scale, and match expression for the standard Blizzard frames",
        args = {
            enabled = {
                type = 'toggle',
                name = "Enabled",
                desc = "When checked, enable the use of UAB action sets for standard Blizzard frames",
                order = 1,
                get = function()
                    return not self.db.profile.disabled
                end,
                set = function(v)
                    self.db.profile.disabled = not v
                    self.core.core.main:FrameGroupsChanged()
                end,
            },
            
            spacing3 = {
                name = " ",
                type = 'header',
                order = 5,
            },
        },
    }
    
    args.args["Player"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Player"], "Player", "Blizzard Player")
    args.args["Player"].disabled = function() return self.db.profile.disabled end
    args.args["Player"].order = 10
    
    args.args["Player Pet"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Player Pet"], "Player Pet", "Blizzard Player Pet")
    args.args["Player Pet"].disabled = function() return self.db.profile.disabled end
    args.args["Player Pet"].order = 20
    
    args.args["Party"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Party"], "Party", "Blizzard Party")
    args.args["Party"].disabled = function() return self.db.profile.disabled end
    args.args["Party"].order = 30
    
    args.args["Target"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Target"], "Target", "Blizzard Target")
    args.args["Target"].disabled = function() return self.db.profile.disabled end
    args.args["Target"].order = 40
    
    args.args["Target of Target"] = self:CreateFrameGroupOption(self.db.profile.framegroups["Target of Target"], "Target of Target", "Blizzard Target of Target")
    args.args["Target of Target"].disabled = function() return self.db.profile.disabled end
    args.args["Target of Target"].order = 50
    
    return args
end

function uabBlizzardFrameGroup:SetupLayouts()
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Player"].layout, self, "Player")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Player Pet"].layout, self, "Player Pet")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Party"].layout, self, "Party")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Target"].layout, self, "Target")
    uabLayoutManager:LayoutChanged(self.db.profile.framegroups["Target of Target"].layout, self, "Target of Target")
end

function uabBlizzardFrameGroup:SetLayoutConfigureOptions(name, args)
    self.core.core.options.args.lookandfeel.args.framegroups.args[self.name].args[name].args.layout.args.configure.args = args
end

